from .oauth import (client, client_fields)
from .firebase_auth import (firebase_auth, FirestoreApp)
def handleauth():
    return "handled"
# from .oauth.models.client_fields import (ClientFieldProps, ClientFields)
# from .models.client import (Client)
# from .firebase import FirestoreApp