genders_data = [
    { "criterionId": 10, "name": 'Homme', "text": 'Male' },
    { "criterionId": 11, "name": 'Femme', "text": 'Female' },
    {
        "criterionId": 20,
        "name": 'Non renseignés à Google',
        "text": 'Undetermined',
        "info": '(il est recommandé de laisser cette option cochée)'
    }
]

ages_data = [
    { "criterionId": 503001, "name": '18-24 ans', "text": '18-24 yo' },
    { "criterionId": 503002, "name": '25-34 ans', "text": '25-34 yo' },
    { "criterionId": 503003, "name": '35-44 ans', "text": '34-44 yo' },
    { "criterionId": 503004, "name": '45-54 ans', "text": '45-54 yo' },
    { "criterionId": 503005, "name": '55-64 ans', "text": '55-64 yo' },
    { "criterionId": 503006, "name": '+64 ans', "text": '64up' },
    {
        "criterionId": 503999,
        "name": 'Non renseignés à Google',
        "text": 'Undetermined',
        "info": '(il est recommandé de laisser cette option cochée)'
    }
]