ads_channel_data = [
    {
        "obj_id": 'search',
        "primary": {
        "icon": 'https://storage.googleapis.com/images.adafri.com/gif/rsa.gif',
        "title": 'Annonces sur le moteur de recherche',
        "description":
            'Vos annonces apparaissent dans le moteur de recherche de Google lorsque des clients potentiels recherchent \
            des mots clés liés à vote activité. Vous ne payez que si une personne interagit avec votre annonce.'
        },
        "secondary": {
        "icon": 'pageview',
        "title": 'Annonces sur le moteur de recherche',
        "description":
            'Vos annonces apparaissent dans le moteur de recherche de Google lorsque des clients potentiels recherchent\
            des mots clés liés à vote activité. Vous ne payez que si une personne interagit avec votre annonce.'
        }
    },
    {
        "obj_id": 'display',
        "primary": {
        "icon": 'https://storage.googleapis.com/images.adafri.com/gif/da.gif',
        "title": 'Annonces avec des visuels',
        "description":
            'Travaillez vos annonces avec votre infographe et importez les simplement pour les remettre à la régie de Google.'
        },
        "secondary": {
        "icon": 'web',
        "title": 'Annonces avec des visuels',
        "description":
            'Travaillez vos annonces avec votre infographe et importez les simplement pour les remettre à la régie de Google.'
        }
    },
    {
        "obj_id": 'native',
        "primary": {
        "icon": 'https://storage.googleapis.com/images.adafri.com/gif/rsda.gif',
        "title": 'Pas d\'infographies ? Optez pour le responsive display',
        "description":
            'Une importation simple de votre logo et d\'images et une adaptation automatique aux formats de Google \
            nécessaires à votre diffusion. Il est possible également de rechercher des images à partir d\'un site web;'
        },
        "secondary": {
        "icon": 'web',
        "title": 'Pas d\'infographies ? Optez pour le responsive display',
        "description":
            'Une importation simple de votre logo et d\'images et une adaptation automatique aux formats de Google \
            nécessaires à votre diffusion. Il est possible également de rechercher des images à partir d\'un site web;'
        }
    }
]